/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic
 *
 * de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.mv.Fg254FunkKriWeissElectronic wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */

package de.bsvrz.kex.tls.osi7.conversion.mv;

import de.bsvrz.dav.daf.main.*;
import de.bsvrz.kex.tls.osi7.*;
import de.bsvrz.kex.tls.osi7.conversion.Fg254Default;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Projektspezifisches Umsetzungsmodul (Fg254FunkKriWeissElectronic) für
 * Funk-KRI von Weiss-Electronik, erweitert um allgemeinen Funktionen sowie
 * Funktionen des Testmusters im Projekt Datenunterzentrale MV/ST
 * ( Kristian Strey )
 *   - Fg254Typ153Id003AtgTlsSecondaryTelefonNummernZuordnungAspParameterIst
 *     Funktion falsch bezeichnet (...Typ153...)
 *   - Ziffern müssen auf TLS-Ebene ASCII codiert werden, betrifft
 *     Fg254Typ153Id003AtgTlsTelefonNummerAspParameterSoll und
 *     Fg254Typ153Id003AtgTlsTelefonNummerAspParameterIst
 *   - für Mehrfachauswertung der DE-Blöcke des Aktionsparametersatzes muss
 *     Leseposition immer wieder auf Anfang gestellt werden,
 *     Fg254Typ153Id003AtgTlsTelefonNummerAspParameterIst und
 *     Fg254Typ153Id003AtgTlsSecondaryTelefonNummernZuordnungAspParameterIst
 *
 *   - FG 254 : Typ: 162 Id: 3, allgemeine Buskonfiguration (alle Weiss KRI,
 *     es wird die ATG der komprimierten Buskonfiguration verwendet)
 *   - Umsetzungsmodule korrigiert / optimiert
 *     Fg254Typ162Id003AtgTlsBusKonfigurationAspParameterIst und
 *     Fg254Typ162Id003AtgTlsBusKonfigurationAspParameterSoll
 *
 *   - FG 254 : Typ: 168 Id: 3, Telefon-Parameter für KRI (nur Testmuster,
 *     es lassen sich maximal 8 Telefonnummern im Block zuordnen)
 */

/**
 * Umsetzungsklasse für FG 254, Typ 166, ... Paramtrierung der KRI von Weiss Electronic
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kniß (HCK)
 * @version $Revision$ / $Date$ / ($Author$)
 */

public class Fg254FunkKriWeissElectronic extends Fg254Default {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Sonstige ---------------------------------------------------------------
	/**
	 * DebugLogger für Debug-Ausgaben
	 */
	private static final Debug debug = Debug.getLogger();
// Private ----------------------------------------------------------------
	/**
	 * Tabelle, in der das Verhalten beim Empfang von Attributgruppen eingetragen wird. Jede Tabellenzeile hat
	 * folgenden Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.parameterSoll", "1", "29", "2", "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Empfangsanmeldung als Empfänger.</li> <li> In der Methode {@link #update} wird für jede zur
	 * Attributgruppen/Aspket-Kombination passende Kombination eine Methode aufgerufen, deren Namen aus dem
	 * Tabelleneintrag wie folgt gebildet wird (hier für den Beispieleintrag):<br>
	 * <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll(ResultData)</code><br> Dabei enthält
	 * <code>ResultData</code> die Daten der zu verarbeitenden ATG/ASP-Kombination.</li> <li> Für jede
	 * Attributguppen/Aspekt Kombination erfolgt eine Empfangsanmeldung als Empfänger. Sind mehrere passende
	 * Eintrage vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch können mehrere
	 * ATG/ASP-Kombinationen in einen De-Block abgebildet werden (z.B. protokollnahe TLS-Attributgruppe und
	 * Attributgruppe eines Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard" (passt
	 * immer). </ul>
	 */
	private final Object[][] davToTls = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsKomprimierteBusKonfiguration", "asp.tlsVorgabe", "254", "166", "3", "typ.deSys"},
			{"atg.tlsTelefonNummer", "asp.parameterSoll", "254", "153", "3", "typ.deSys"},
			{"atg.tlsSecondaryTelefonNummernZuordnung", "asp.parameterSoll", "254", "153", "3", "typ.deSys"},
			{"atg.tlsAbrufTelefonNummer", "asp.tlsAbruf", "254", "152", "3", "typ.deSys"},
			{"atg.tlsAbrufSecondaryTelefonNummernZuordnung", "asp.tlsAbruf", "254", "152", "3", "typ.deSys"},
			// [KS] allgemeine Buskonfiguration und Telefonparameter für den Test-KRI
			{"atg.tlsBusKonfiguration", "asp.tlsVorgabe", "254", "162", "3", "typ.deSys"},
			{"atg.tlsTelefonParameter", "asp.parameterSoll", "254", "168", "3", "typ.deSys"}
	};

	/**
	 * Tabelle, in der das Verhalten beim Empfang von TLS De-Blöcken eingetragen wird. Jede Tabellenzeile hat
	 * folgenden Aufbau:
	 * <pre>
	 * "Attributgruppe (Pid)", "Aspekt (Pid)", "Fg", "Typ", "Id", "ObjektTyp"
	 * z.B.
	 * "atg.tlsGloKanalSteuerung", "asp.tlsAntwort", "1", "29", "2", "typ.deLve"
	 * </pre>
	 * Die Auswertung erfolgt dabei wie folgt: <ul> <li> Für jede Attributguppen/Aspekt Kombination erfolgt eine
	 * Sendeanmeldung als Quelle.</li> <li> In der Methode {@link #processDeBlock} wird für jede zum De-Block
	 * passende Fg/Id/Typ-Kombination eine Methode aufgerufen, deren Namen aus dem Tabelleneintrag wie folgt
	 * gebildet wird (hier für den Beispieleintrag):<br> <code>Fg001Typ029Id002AtgTlsGloKanalSteuerungAspTlsAntwort(DeBlock,
	 * Data, DataDescription)</code><br> Dabei enthält der <code>deBlock</code> die Daten des zu verarbeitenden
	 * De-Blocks und <code>data</code> ein Datenobjekt der passenden Attributgruppen/Aspekt-Kombination, welchem
	 * direkt die Inhalte des De-Blocks zugewiesen werden können.<code>DataDescription</code> enthält die passende
	 * {@link DataDescription} für die zu sendenden Daten.</li> <li> Für jede
	 * Attributguppen/Aspekt Kombination erfolgt eine Sendeanmeldung als Quelle. Sind mehrere passende Eintrage
	 * vorhanden, werden entsprechend mehrere Methodenaufrufe durchgeführt. Dadurch kann ein De-Block mehrfach
	 * ausgewert und an verschiedene Attributgruppen weitergerleitet werden (z.B. protokollnahe TLS-Attributgruppe
	 * und Attributgruppe eine Fachmodells):</li> <li> Ein Eintrag von "255" entspricht dabei einer "Wildcard"
	 * (passt immer). </ul>
	 */
	private final Object[][] tlsToDav = new Object[][]{
			// Attributgruppe, Aspekt, Fg, Typ, Id, ObjektTyp
			{"atg.tlsKomprimierteBusKonfiguration", "asp.tlsZustand", "254", "166", "3", "typ.deSys"},
			{"atg.tlsTelefonNummer", "asp.parameterIst", "254", "153", "3", "typ.deSys"},
			{"atg.tlsSecondaryTelefonNummernZuordnung", "asp.parameterIst", "254", "153", "3", "typ.deSys"},
			// [KS] allgemeine Buskonfiguration und Telefonparameter für den Test-KRI
			{"atg.tlsBusKonfiguration", "asp.tlsZustand", "254", "162", "3", "typ.deSys"},
			{"atg.tlsTelefonParameter", "asp.parameterIst", "254", "168", "3", "typ.deSys"}
	};

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt ein Objekt vom Typ Fg254FunkKriWeissElectronic
	 *
	 * @param daf DaV
	 * @param eak Eak
	 * @param de De
	 */
	public Fg254FunkKriWeissElectronic(ClientDavInterface daf, Eak eak, De de) {
		super(daf, eak, de);
		subscribeAllData(tlsToDav, davToTls);
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * FG 254 : Typ: 152 Id: 3 Name: SYS-Abruf Aktionsparameter (Abruf) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ152Id003AtgTlsAbrufSecondaryTelefonNummernZuordnungAspTlsAbruf(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_ = 13; // Weiss-Electronic.
		int _Version_ = 1;
		int _Parametertyp_ = 5;
		int _MasterAdresse_;
		int _SlaveAdresse_;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		_MasterAdresse_ = data.getUnscaledValue("MasterAdresse").intValue();
		_SlaveAdresse_ = data.getUnscaledValue("SlaveAdresse").intValue();

		// Länge ermitteln...
		int length = 6;

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_Version_);
		deApplicationData.writeIntAsByte(_Parametertyp_);
		deApplicationData.writeIntAsByte(_MasterAdresse_);
		deApplicationData.writeIntAsByte(_SlaveAdresse_);

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 152, 3, 0); // immer an DE o = KRI)
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 254 : Typ: 152 Id: 3 Name: SYS-Abruf Aktionsparameter (Abruf) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ152Id003AtgTlsAbrufTelefonNummerAspTlsAbruf(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_ = 13; // Weiss-Electronic.
		int _Version_ = 1;
		int _Parametertyp_ = 2;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		if (data.getUnscaledValue("Abruf").getState() != null) {
			// [KS] der Status kann nicht mit einem String verglichen werden, sein Name schon
			//                  if (!data.getUnscaledValue("Abruf").getState().equals("Ja")) {
			if (!data.getUnscaledValue("Abruf").getState().getName().equals("Ja")) {
				return deBlock; // Nichts tun, da kein Abruf erfolgen soll.
			}
		}
		// Länge ermitteln...
		int length = 4;

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_Version_);
		deApplicationData.writeIntAsByte(_Parametertyp_);
		deApplicationData.writeIntAsByte(_de.getDeChannel()); // Listenplatz ist DeKanal.

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 152, 3, 0); // immer an DE o = KRI)
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	// [KS] Funktionsname korrigiert, Typ 153

	/**
	 * FG 254 : Typ: 153 Id: 3 Name: SYS-Zuweisung Aktionsparameter (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 153 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg254Typ153Id003AtgTlsSecondaryTelefonNummernZuordnungAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;
		// [KS] für Mehrfachauswertung des Aktionsparametersatzes muss
		//      Leseposition immer wieder auf Anfang gestellt werden
		deBlock.getDeData().rewindReadPosition();

		// Herstellercode/Version/Parametertyp-Bytes aus Telegramm -----------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Version_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Parametertyp_ = deBlock.getDeData().readUnsignedByteAsInt();

		if ((_Herstellercode_ == 13) && (_Version_ == 1) && (_Parametertyp_ == 5)) {
			// Telegramm enthält Secandary-TelefonNummernZuordnung für diesen Listenplatz, Listenplatz ergibt
			// sich aus DeKanal. De-kanal wurde entsprechend EakUmsetzungsmodul von De=0 auf De=Listenplatz
			// geändert.

			int _MasterAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			int _SlaveAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			int _ListenPlatz_ = deBlock.getDeData().readUnsignedByteAsInt();
			int _AnzahlAnrufVersuche_ = deBlock.getDeData().readUnsignedByteAsInt();

			data.getUnscaledValue("MasterAdresse").set(_MasterAdresse_);
			data.getUnscaledValue("SlaveAdresse").set(_SlaveAdresse_);
			data.getUnscaledValue("ListenPlatz").set(_ListenPlatz_);
			data.getUnscaledValue("AnzahlAnrufVersuche").set(_AnzahlAnrufVersuche_);
		}
		else {
			// Telegramm enthält TelefonNummern --> Umsetzung erfolgt in eigener Methode
			return resultData;
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(),
		                            data);
		return resultData;
	}

	/**
	 * FG 254 : Typ: 153 Id: 3 Name: SYS-Zuweisung Aktionsparameter (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ153Id003AtgTlsSecondaryTelefonNummernZuordnungAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_ = 13; // Weiss-Electronic.
		int _Version_ = 1;
		int _Parametertyp_ = 5;
		int _MasterAdresse_;
		int _SlaveAdresse_;
		int _ListenPlatz_;
		int _AnzahlAnrufVersuche_;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		_MasterAdresse_ = data.getUnscaledValue("MasterAdresse").intValue();
		_SlaveAdresse_ = data.getUnscaledValue("SlaveAdresse").intValue();

		//		if (data.getUnscaledValue("ListenPlatz").getState() == null) {
		//			_ListenPlatz_ = data.getUnscaledValue("ListenPlatz").intValue();
		//		}
		//		else {
		//			_ListenPlatz_ = 0;
		//		}

		_ListenPlatz_ = _de.getDeChannel(); // Listenplatz ist immer De-Nummer;

		if (data.getUnscaledValue("AnzahlAnrufVersuche").getState() == null) {
			_AnzahlAnrufVersuche_ = data.getUnscaledValue("AnzahlAnrufVersuche").intValue();
		}
		else {
			_AnzahlAnrufVersuche_ = 0;
		}

		// Länge ermitteln...
		int length = 7;

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_Version_);
		deApplicationData.writeIntAsByte(_Parametertyp_);
		deApplicationData.writeIntAsByte(_MasterAdresse_);
		deApplicationData.writeIntAsByte(_SlaveAdresse_);
		deApplicationData.writeIntAsByte(_ListenPlatz_);
		deApplicationData.writeIntAsByte(_AnzahlAnrufVersuche_);

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 153, 3, 0); // immer an DE o = KRI)
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 254 : Typ: 153 Id: 3 Name: SYS-Zuweisung Aktionsparameter (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg254Typ153Id003AtgTlsTelefonNummerAspParameterIst(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;
		// [KS] für Mehrfachauswertung des Aktionsparametersatzes muss
		//      Leseposition immer wieder auf Anfang gestellt werden
		deBlock.getDeData().rewindReadPosition();

		// Herstellercode/Version/Parametertyp-Bytes aus Telegramm -----------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Version_ = deBlock.getDeData().readUnsignedByteAsInt();
		int _Parametertyp_ = deBlock.getDeData().readUnsignedByteAsInt();

		if ((_Herstellercode_ == 13) && (_Version_ == 1) && (_Parametertyp_ == 2)) {
			// Telegramm enthält Telefonnummer für diesen Listenplatz, Listenplatz ergibt sich aus DeKanal.
			// De-kanal wurde entsprechend EakUmsetzungsmodul von De=0 auf De=Listenplatz geändert, so dass
			// dieser hier nicht mehr ausgewertet werden muss.
			deBlock.getDeData().readUnsignedByteAsInt(); // Listenplatz überlesen.

			// [KS] Der KRI liefert zur Zeit ein falsches Telegramm, ES FEHLT DIE LÄNGE
			//      der Telefonnummer (!) - > Workaround: Leseposition zurückstetzen und
			//      Länge = Anzahl Datenbytes - (Herstellercode+Version+Parametertyp+Listenplatz)
			//            int count = deBlock.getDeData().length() - 4;

			int count = deBlock.getDeData().readUnsignedByteAsInt(); // Anzahl der Ziffern der Telefonnummer

			debug.finer(this.getClass().getName() + ": analysiere Telefonnummer: Länge=" + count);

			// [KS] Ziffern werden auf TLS-Ebene ASCII codiert gespeichert
			//      Konvertierung '0'..'9' -> 0..9 eingefügt
			data.getUnscaledArray("Ziffer").setLength(count);
			for (int i = 0; i < count; i++) {
				char ziffer = (char) (deBlock.getDeData().readUnsignedByteAsInt() & 0xFF);
				int _Ziffer_ = Character.digit(ziffer, 10);
				debug.finer(this.getClass().getName() + ": analysiere Telefonnummer, Ziffer " + i + ": '" +
				            ziffer + "' -> " + _Ziffer_ + " (ziffer-'0')=" + (int) (ziffer - '0'));
				data.getUnscaledArray("Ziffer").getValue(i).set(_Ziffer_);
			}
		}
		else {
			// Telegramm enthält Secandary-TelefonNummernZuordnung --> Umsetzung erfolgt in eigener Methode
			return resultData;
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(),
		                            data);
		return resultData;
	}

	/**
	 * FG 254 : Typ: 153 Id: 3 Name: SYS-Zuweisung Aktionsparameter (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ153Id003AtgTlsTelefonNummerAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_ = 13; // Weiss-Electronic.
		int _Version_ = 1;
		int _Parametertyp_ = 2;

		int[] _Ziffer_;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		// [KS] Ziffern sind auf TLS-Ebene ASCII codiert zu speichern
		//      Konvertierung 0..9 -> '0'..'9' eingefügt
		int _anzZiffern_ = data.getArray("Ziffer").getLength();
		_Ziffer_ = new int[_anzZiffern_];
		for (int i = 0; i < _anzZiffern_; i++) {
			_Ziffer_[i] = '0' + data.getUnscaledArray("Ziffer").getValue(i).intValue();
		}

		// DeBlock als TlsData aufbauen...

		// Länge ermitteln...
		int length = _anzZiffern_ + 5;

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_Version_);
		deApplicationData.writeIntAsByte(_Parametertyp_);
		deApplicationData.writeIntAsByte(_de.getDeChannel()); // De Nummer ist gleichzeitig der Listenplatz.
		deApplicationData.writeIntAsByte(_anzZiffern_);
		for (int i = 0; i < _anzZiffern_; i++) {
			deApplicationData.writeIntAsByte(_Ziffer_[i]);
		}

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 153, 3, 0); // immer an DE o = KRI)
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 254 : Typ: 162 Id: 3 Name: SYS-Bus-Konfiguration (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 162 ]
	 * Byte 4           ...
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ162Id003AtgTlsBusKonfigurationAspTlsVorgabe(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_;
		int _ComPort_[];
		int _MasterAdresse_[];
		int _SlaveAdresse_[][];

		int _anzBusKonfigurationen_;
		int _anzSlaveAdressen_;

		// Datensatz holen
		Data data = result.getData();

		_anzBusKonfigurationen_ = data.getArray("BusKonfiguration").getLength();
		_ComPort_ = new int[ _anzBusKonfigurationen_ ];
		_MasterAdresse_ = new int[ _anzBusKonfigurationen_ ];
		_SlaveAdresse_ = new int[ _anzBusKonfigurationen_ ][];

		_Herstellercode_ = data.getUnscaledValue("Hersteller").intValue();

		// Buskonfiguration analysieren (für TLS-Längenberechnung...)
		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			Data eintrag = data.getArray("BusKonfiguration").getItem(i);
			_ComPort_[i] = eintrag.getUnscaledValue("ComPort").intValue();
			_ComPort_[i] += (eintrag.getUnscaledValue("ComTyp").intValue() << 7);
			_MasterAdresse_[i] = eintrag.getUnscaledValue("MasterAdresse").intValue();
			_anzSlaveAdressen_ = eintrag.getArray("SlaveAdresse").getLength();
			_SlaveAdresse_[i] = new int[ _anzSlaveAdressen_ ];
			for (int j = 0; j < _anzSlaveAdressen_; j++) {
				_SlaveAdresse_[i][j] = eintrag.getUnscaledArray("SlaveAdresse").intValue(j);
			}
		}

		// DEBUG Ausgaben
		StringBuffer sb = new StringBuffer();
		sb.append("\n_anzBusKonfigurationen_= " + _anzBusKonfigurationen_);
		sb.append("\n_Com_Port_ = ");
		for (int i = 0; i < _ComPort_.length; i++) {
			sb.append(_ComPort_[i]).append(' ');
		}
		sb.append("\n_MasterAdresse_ = ");
		for (int i = 0; i < _MasterAdresse_.length; i++) {
			sb.append(_MasterAdresse_[i]).append(' ');
		}
		sb.append("\n_SlaveAdresse_ = ");
		for (int i = 0; i < _SlaveAdresse_.length; i++) {
			sb.append("\n\t");
			for (int j = 0; j < _SlaveAdresse_[i].length; j++) {
				sb.append(_SlaveAdresse_[i][j]).append(' ');
			}
		}
		debug.finest(this.getClass().getName() + " ATG Daten analysiert: " + sb.toString());

		// DeBlock als TlsData aufbauen...

		// Länge ermitteln (Anzahl Tabellenzeilen * 3 Spalten)...
		int _anzahlTabellenZeilen_ = 0;
		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			_anzahlTabellenZeilen_ += _SlaveAdresse_[i].length;
		}
		int length = 2 + _anzahlTabellenZeilen_ * 3;

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_anzahlTabellenZeilen_);
		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			for (int j = 0; j < _SlaveAdresse_[i].length; j++) {
				// Tabellenzeile schreiben
				deApplicationData.writeIntAsByte(_ComPort_[i]);
				deApplicationData.writeIntAsByte(_MasterAdresse_[i]);
				deApplicationData.writeIntAsByte(_SlaveAdresse_[i][j]);
			}
		}

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(),
			                      162, 3, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	// [KS] >>>>>>>>>> Beginn der zusätzlich implementierten Funktionen

	/**
	 * FG 254 : Typ: 162 Id: 3 Name: SYS-Bus-Konfiguration (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 162 ]
	 * Byte 4           ...
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg254Typ162Id003AtgTlsBusKonfigurationAspTlsZustand(DeBlock deBlock,
	                                                                      Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Anzahl Tabellenzeilen aus Telegramm ------------------------------------------------------------------------------
		int count = deBlock.getDeData().readUnsignedByteAsInt();
		int COM_PORT_ID[] = {1, 2, 4, 6};
		int _ComPorts_[] = {0, 0, 0, 0};
		int _MasterAdressen_[] = {0, 0, 0, 0};
		int _SlaveAdressenCount_[] = {0, 0, 0, 0};
		int _SlaveAdressen_[][] =
				{{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
						, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
						, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
						, {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
				};

		// Daten aus DE-Block lesen und analysieren
		for (int i = 0; i < count; i++) {
			int _ComPort_ = deBlock.getDeData().readUnsignedByteAsInt();
			int _MasterAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			int _SlaveAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			int index = -1;
			for (int j = 0; j < COM_PORT_ID.length; j++) {
				if (_ComPort_ == COM_PORT_ID[j]) {
					index = j;
				}
				;
			}
			if (index != -1) {
				_ComPorts_[index] = COM_PORT_ID[index];
				_MasterAdressen_[index] = _MasterAdresse_;
				_SlaveAdressen_[index][_SlaveAdressenCount_[index]] = _SlaveAdresse_;
				if (_SlaveAdressenCount_[index] < 15) {
					_SlaveAdressenCount_[index]++;
				}
			}
		}

		// DEBUG Ausgaben
		StringBuffer sb = new StringBuffer();
		sb.append("\n_Com_Ports_= ");
		for (int i = 0; i < _ComPorts_.length; i++) {
			sb.append(_ComPorts_[i]).append(' ');
		}
		sb.append("\n_MasterAdressen_ = ");
		for (int i = 0; i < _MasterAdressen_.length; i++) {
			sb.append(_MasterAdressen_[i]).append(' ');
		}
		sb.append("\n_SlaveAdressenCount_ = ");
		for (int i = 0; i < _SlaveAdressenCount_.length; i++) {
			sb.append(_SlaveAdressenCount_[i]).append(' ');
		}
		sb.append("\n_SlaveAdressen_ = ");
		for (int i = 0; i < _SlaveAdressen_.length; i++) {
			sb.append("\n\t");
			for (int j = 0; j < _SlaveAdressen_[i].length; j++) {
				sb.append(_SlaveAdressen_[i][j]).append(' ');
			}
		}
		debug.finest(this.getClass().getName() + " TLS Daten analysiert: " + sb.toString());

		// Daten in ATG-Struktur schreiben

		// Anzahl notwendiger Buskonfigurationseinträge für die ATG ermitteln
		int _ComPortCount_ = 0;
		for (int i = 0; i < _ComPorts_.length; i++) {
			if (_ComPorts_[i] == COM_PORT_ID[i]) {
				_ComPortCount_++;
			}
		}
		data.getArray("BusKonfiguration").setLength(_ComPortCount_);
		// Einträge in die ATG schreiben
		int idxEintrag = 0;
		for (int i = 0; i < _ComPorts_.length; i++) {
			if (_ComPorts_[i] == COM_PORT_ID[i]) {
				Data eintrag = data.getArray("BusKonfiguration").getItem(idxEintrag);
				eintrag.getUnscaledValue("ComPort").set(_ComPorts_[i] & 0x07);
				eintrag.getUnscaledValue("ComTyp").set(_ComPorts_[i] >> 7);
				eintrag.getUnscaledValue("MasterAdresse").set(_MasterAdressen_[i]);
				eintrag.getArray("SlaveAdresse").setLength(_SlaveAdressenCount_[i]);
				for (int j = 0; j < _SlaveAdressenCount_[i]; j++) {
					eintrag.getUnscaledArray("SlaveAdresse").getValue(j)
							.set(_SlaveAdressen_[i][j]);
				}
				idxEintrag++;
			}
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(),
		                            data);
		return resultData;
	}

	/**
	 * FG 254 : Typ: 166 Id: 3 Name: SYS-Komprimierte-Bus-Konfiguration (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ166Id003AtgTlsKomprimierteBusKonfigurationAspTlsVorgabe(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_;
		int _ComPort_[];
		int _MasterAdresse_[];
		int _SlaveAdresse_[][];

		int _anzBusKonfigurationen_;
		int _anzSlaveAdressen_;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		_anzBusKonfigurationen_ = data.getArray("BusKonfiguration").getLength();
		_ComPort_ = new int[_anzBusKonfigurationen_];
		_MasterAdresse_ = new int[_anzBusKonfigurationen_];
		_SlaveAdresse_ = new int[_anzBusKonfigurationen_][];

		_Herstellercode_ = data.getUnscaledValue("Hersteller").intValue();

		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			_ComPort_[i] = data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("ComPort").intValue();
			_ComPort_[i] += (data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("ComTyp").intValue() << 7);
			_MasterAdresse_[i] = data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("MasterAdresse").intValue();

			_anzSlaveAdressen_ = data.getArray("BusKonfiguration").getItem(i).getArray("SlaveAdresse").getLength();
			_SlaveAdresse_[i] = new int[_anzSlaveAdressen_];
			for (int j = 0; j < _anzSlaveAdressen_; j++) {
				_SlaveAdresse_[i][j] = data.getArray("BusKonfiguration").getItem(i).getUnscaledArray("SlaveAdresse").intValue(j);
			}
		}

		// DeBlock als TlsData aufbauen...

		// Länge ermitteln...
		int length = 2;
		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			length += _SlaveAdresse_[i].length + 3;
		}

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_anzBusKonfigurationen_);
		for (int i = 0; i < _anzBusKonfigurationen_; i++) {
			deApplicationData.writeIntAsByte(_ComPort_[i]);
			deApplicationData.writeIntAsByte(_MasterAdresse_[i]);
			deApplicationData.writeIntAsByte(_SlaveAdresse_[i].length);
			for (int j = 0; j < _SlaveAdresse_[i].length; j++) {
				deApplicationData.writeIntAsByte(_SlaveAdresse_[i][j]);
			}
		}

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(), 166, 3, 0); // immer an DE o = KRI)
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	/**
	 * FG 254 : Typ: 166 Id: 3 Name: SYS-Komprimierte-Bus-Konfiguration (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 166 ]
	 * Byte 4
	 * Byte 5
	 * Byte 6
	 * Byte 7
	 * Byte 8
	 * Byte 9
	 * Byte 10
	 * Byte 11
	 * </pre>
	 * Inhalt Byte: xxx<br> yyy<br>
	 *
	 * <pre>
	 * xxx
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg254Typ166Id003AtgTlsKomprimierteBusKonfigurationAspTlsZustand(DeBlock deBlock, Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Buskonfiguration aus Telegramm ------------------------------------------------------------------------------
		int count = deBlock.getDeData().readUnsignedByteAsInt();
		int _ComPort_;
		int _MasterAdresse_;

		data.getArray("BusKonfiguration").setLength(count);

		for (int i = 0; i < count; i++) {
			_ComPort_ = deBlock.getDeData().readUnsignedByteAsInt();
			_MasterAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();

			data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("ComPort").set(_ComPort_ & 0x07);
			data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("ComTyp").set(_ComPort_ >> 7);
			data.getArray("BusKonfiguration").getItem(i).getUnscaledValue("MasterAdresse").set(_MasterAdresse_);

			int countMasterAdresse = deBlock.getDeData().readUnsignedByteAsInt();
			int _SlaveAdresse_;

			data.getArray("BusKonfiguration").getItem(i).getArray("SlaveAdresse").setLength(countMasterAdresse);

			for (int j = 0; j < countMasterAdresse; j++) {
				_SlaveAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
				data.getArray("BusKonfiguration").getItem(i).getUnscaledArray("SlaveAdresse").getValue(j).set(_SlaveAdresse_);
			}
		}


		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(), data);
		return resultData;
	}

	/**
	 * FG 254 : Typ: 168 Id: 3 Name: Telefon-Parameter für KRI (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 168 ]
	 * Byte 4           ...
	 * </pre>
	 *
	 * @param deBlock         Der zu bearbeitende De-Block.
	 * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
	 * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
	 *
	 * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
	 */
	public ResultData Fg254Typ168Id003AtgTlsTelefonParameterAspParameterIst(DeBlock deBlock,
	                                                                        Data data, DataDescription dataDescription) {
		ResultData resultData = null;

		// Herstellercode-Byte aus Telegramm ---------------------------------------------------------------------------
		int _Herstellercode_ = deBlock.getDeData().readUnsignedByteAsInt();
		data.getUnscaledValue("Hersteller").set(_Herstellercode_);

		// Parameterzeilen aus Telegramm ------------------------------------------------------------------------------
		int count = deBlock.getDeData().readUnsignedByteAsInt();
		int _MasterAdresse_;
		int _SlaveAdresse_;
		int _RufWiederholungen_;

		data.getArray("TelefonParameter").setLength(count);

		for (int i = 0; i < count; i++) {
			_MasterAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			_SlaveAdresse_ = deBlock.getDeData().readUnsignedByteAsInt();
			_RufWiederholungen_ = deBlock.getDeData().readUnsignedByteAsInt();

			data.getArray("TelefonParameter").getItem(i).getUnscaledValue("MasterAdresse").set(
					_MasterAdresse_);
			data.getArray("TelefonParameter").getItem(i).getUnscaledValue("SlaveAdresse").set(
					_SlaveAdresse_);
			data.getArray("TelefonParameter").getItem(i).getUnscaledValue("RufWiederholungen").set(
					_RufWiederholungen_);

			int countRufNummer = deBlock.getDeData().readUnsignedByteAsInt();
			StringBuffer _RufNummer_ = new StringBuffer();

			for (int j = 0; j < countRufNummer; j++) {
				char _char_ = (char) (deBlock.getDeData().readUnsignedByteAsInt() & 0xFF);
				_RufNummer_.append(_char_);
			}
			data.getArray("TelefonParameter").getItem(i).getTextValue("RufNummer").setText(
					_RufNummer_.toString());
		}

		//Ergebnisdatensatz erzeugen...
		resultData = new ResultData(_de.getDe(), dataDescription, _eakConversionModul.getTimeStamp(),
		                            data);
		return resultData;
	}

	/**
	 * FG 254 : Typ: 168 Id: 3 Name: Telefon-Parameter für KRI  (Abruf / Antwort) <br> TLS-Datenbeschreibung
	 * (Herstellerspezifisch WEISS-ELECTRONIC):
	 * <pre>
	 * Position         Bezeichnung                                Erläuterung
	 * Byte 1           Länge DE-Block                             Länge des folgenden DE-Blocks [  ]
	 * Byte 2           Daten-Endgeräte-Kanal                      [0 = Systemsteuerung des KRI]
	 * Byte 3           Typ der DE-Daten                           [ 168 ]
	 * Byte 4           ...
	 * </pre>
	 *
	 * @param result Der vom Datenverteiler empgangene Datensatz, der in einen De-Block konvertiert wird.
	 *
	 * @return De-Block mit den konvertierten Daten.
	 */
	public DeBlock Fg254Typ168Id003AtgTlsTelefonParameterAspParameterSoll(ResultData result) {
		DeBlock deBlock = null;

		// Datenblock beseht aus folgenden Inhalten:
		int _Herstellercode_;
		int _MasterAdresse_[];
		int _SlaveAdresse_[];
		int _RufWiederholungen_[];
		String _RufNummer_[];
		int _anzParameterZeilen_;

		// Umsetzung
		Data data = result.getData(); // Datensatz holen

		_Herstellercode_ = data.getUnscaledValue("Hersteller").intValue();

		_anzParameterZeilen_ = data.getArray("TelefonParameter").getLength();
		_MasterAdresse_ = new int[ _anzParameterZeilen_ ];
		_SlaveAdresse_ = new int[ _anzParameterZeilen_ ];
		_RufWiederholungen_ = new int[ _anzParameterZeilen_ ];
		_RufNummer_ = new String[ _anzParameterZeilen_ ];

		for (int i = 0; i < _anzParameterZeilen_; i++) {
			_MasterAdresse_[i] = data.getArray("TelefonParameter").getItem(i).getUnscaledValue(
					"MasterAdresse").intValue();
			_SlaveAdresse_[i] = data.getArray("TelefonParameter").getItem(i).getUnscaledValue(
					"SlaveAdresse").intValue();
			_RufWiederholungen_[i] = data.getArray("TelefonParameter").getItem(i).getUnscaledValue(
					"RufWiederholungen").intValue();
			_RufNummer_[i] = data.getArray("TelefonParameter").getItem(i).getTextValue("RufNummer").
					getValueText();
		}

		// DeBlock als TlsData aufbauen...

		// Länge ermitteln...
		int length = 2;
		for (int i = 0; i < _anzParameterZeilen_; i++) {
			length += _RufNummer_[i].length() + 4;
		}

		//...und De-Block erstellen.

		TlsData deApplicationData = new TlsData(length);
		deApplicationData.writeIntAsByte(_Herstellercode_);
		deApplicationData.writeIntAsByte(_anzParameterZeilen_);
		for (int i = 0; i < _anzParameterZeilen_; i++) {
			deApplicationData.writeIntAsByte(_MasterAdresse_[i]);
			deApplicationData.writeIntAsByte(_SlaveAdresse_[i]);
			deApplicationData.writeIntAsByte(_RufWiederholungen_[i]);
			deApplicationData.writeIntAsByte(_RufNummer_[i].length());
			for (int j = 0; j < _RufNummer_[i].length(); j++) {
				deApplicationData.writeIntAsByte(_RufNummer_[i].charAt(j));
			}
		}

		// DeBlock erzeugen
		try {
			debug.fine("Data:\n" + data + "\n" + "deApplicationData: " + deApplicationData.toString());
			deBlock = new DeBlock(deApplicationData, _de.getEak().getSm().getOsi7Adress(), _de.getFg(),
			                      168, 3, _de.getDeChannel());
		}
		catch (IllegalOsi7TelegramStructureException e) {
			debug.warning("Fehler beim Erzeugen eines De-Blocks zum Versand: " + e);
		}
		return deBlock;
	}

	// [KS] <<<<<<<<<< Ende der Erweiterungen
}
