/*
 * Copyright (c) 2015 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.bsvrz.kex.tls.osi7.conversion.Fg004Default
 *
 * de.bsvrz.kex.tls.osi7.conversion.Fg004Default is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with de.bsvrz.kex.tls.osi7.conversion.Fg004Default.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Diese Datei ist Teil von de.bsvrz.kex.tls.osi7.conversion.Fg004Default.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default ist Freie Software: Sie können es unter den Bedingungen
 *  der GNU General Public License, wie von der Free Software Foundation,
 *  Version 3 der Lizenz oder (nach Ihrer Wahl) jeder späteren
 *  veröffentlichten Version, weiterverbreiten und/oder modifizieren.
 *
 *  de.bsvrz.kex.tls.osi7.conversion.Fg004Default wird in der Hoffnung, dass es nützlich sein wird, aber
 *  OHNE JEDE GEWÄHRLEISTUNG, bereitgestellt; sogar ohne die implizite
 *  Gewährleistung der MARKTFÄHIGKEIT oder EIGNUNG FÜR EINEN BESTIMMTEN ZWECK.
 *  Siehe die GNU General Public License für weitere Details.
 *
 *  Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 *  Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * An der Krautwiese 37
 * D-53783 Eitorf
 * +49 (0)2243 8464 193
 * info@inovat.de
 * www.inovat.de
 */


package de.bsvrz.kex.tls.osi7.conversion.nw.dwista;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.IntegerValueState;
import de.bsvrz.kex.tls.osi7.De;
import de.bsvrz.kex.tls.osi7.DeBlock;
import de.bsvrz.kex.tls.osi7.Eak;
import de.bsvrz.kex.tls.osi7.TlsApplicationLayer;
import de.bsvrz.kex.tls.osi7.conversion.Fg004DefaultMitAusschaltCode0;
import de.bsvrz.sys.funclib.debug.Debug;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

//~ NICHT JDK IMPORTE =========================================================

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Umsetungsmodul für die FG 4, Standard (alle Typen der FG 4 gemäß TLS 2002) Wird standardmäßig für alle FG 4 De verwendet, bei denen keine spezielle
 * Umsezungsklasse in der Konfiguration angegeben wurde. Erweiterung: unterstützt auch nachgelieferte Telegramme (Standard ID+32)
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @version $Revision$ / $Date$ / ($Author$)
 */
public class Fg004Tls2012 extends Fg004DefaultMitAusschaltCode0 {

    private static final Debug debug = Debug.getLogger();

    /**
     * Erzeugt ein Objekt vom Typ Fg004Tls2012
     *
     * @param daf DaV
     * @param eak Eak
     * @param de  De
     */
    public Fg004Tls2012(ClientDavInterface daf, Eak eak, De de) {
        super(daf, eak, de);
    }

    //~ METHODEN ==============================================================

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * FG 4 : Typ: 1 Id: 1 Name: DE-Fehler (Antwort). Ergänzt globale Behandlung De-Fehler in {@link #Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort}. Beim
     * Empfang einer De-Gutmeldung wird die De initialisiert, indem
     * <ul> <li>WZG-kanalsteuerung {@link #Fg255Typ029Id002AtgTlsGloKanalSteuerungAspParameterSoll} TBD: Folgende
     * Einstellung bei FG 4 nach DE-Gutmeldung versenden (aber wie werden die verschiedenen Bauformaen, TLS-Stände
     * konfiguriert ??? {"atg.tlsWzgBetriebsArt", "asp.parameterSoll", "4", "17", "2", "typ.deWzg"},
     * {"atg.tlsWzgWvzGrundEinstellung", "asp.parameterSoll", "4", "32", "3", "typ.deWzg"}, {"atg.tlsWzgGrundEinstellung",
     * "asp.parameterSoll", "4", "33", "3", "typ.deWzg"}, {"atg.tlsWzgGrundProgramm", "asp.parameterSoll", "4", "42", "3",
     * "typ.deWzg"}, {"atg.tlsWzgCodeListe", "asp.parameterSoll", "4", "43", "3", "typ.deWzg"}, </li></ul> versendet werden.
     *
     * @param deBlock         Der zu bearbeitende De-Block.
     * @param data            Der passende Datensatz für die Ziel Attributgruppe/Aspekt-Kombination.
     * @param dataDescription Die Datenbeschreibung für die Attributgruppe/Aspekt-Kombination.
     * @return Ergebnisdatensatz mit den vom DeBlock in die entsprechende Attributgruppe konvertierten Daten.
     */
    public ResultData Fg004Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(DeBlock deBlock, Data data, DataDescription dataDescription) {
        ResultData resultData = super.Fg255Typ001Id001AtgTlsGloDeFehlerAspTlsAntwort(deBlock, data, dataDescription);
        IntegerValueState DEFehlerStatus = data.getUnscaledValue("DEFehlerStatus").getState();

        if(DEFehlerStatus != null) {
            debug.fine("DEFehlerStatus: " + DEFehlerStatus.getName());

            if(DEFehlerStatus.getName().equals("ok")) {
                if(_de.isCluster()) // Clusterkanal
                {
                    sendInitialDeBlock(deBlock.getKey(), 17, 2); // BetriebsArt senden, nur an Clusterkanal
                }
                else // kein Clusterkanal
                {
                    sendInitialDeBlock(deBlock.getKey(), 29, 2); // Kanalsteuerung senden, nicht an Clusterkanal

                    sendInitialDeBlock(deBlock.getKey(), 33, 3); // WzgGrundEinstellung senden
                }
            }
            else {
                List<De> deList = new ArrayList<>();
                deList.add(_de);
                TlsApplicationLayer.writeNoDataToUnreachableDe(deList, _eakConversionModul.getTimeStamp(), -1, "atg.tlsGloDeFehler");
            }
        }

        return resultData;
    }

    @Override
    public List<DeBlock> Fg004Typ032Id003AtgTlsWzgWvzGrundEinstellungAspParameterSoll(ResultData result) {
        debug.fine("Ignoriere Typ032/Id003/AtgTlsWzgWvzGrundEinstellung/AspParameterSoll da in Tls 2012 entfernt");
        return Collections.emptyList();
    }
}
